package uk.co.trinitylogic.skyfall.dao;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.Query;
import org.hibernate.SessionFactory;
import org.hibernate.search.FullTextSession;
import org.hibernate.search.Search;
import org.hibernate.search.query.dsl.QueryBuilder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import uk.co.trinitylogic.skyfall.form.Advice;
import uk.co.trinitylogic.skyfall.form.AdviceComparator;
import uk.co.trinitylogic.skyfall.form.Service;

@Repository
@Transactional
public class AdviceDAOImpl implements AdviceDAO {

	protected final Log logger = LogFactory.getLog(getClass());

	@Autowired
	private SessionFactory sessionFactory;

	@Override
	public void addAdvice(Advice advice) {
		logger.info("Adding a new advice: " + advice);
		long startTime = System.nanoTime();
		sessionFactory.getCurrentSession().save(advice);
		long endTime = System.nanoTime();
		long duration = (endTime - startTime) / 1000000;
		logger.info("Added a new advice: " + advice + " Time: " + duration
				+ "ms");
	}

	@Override
	public void removeAdvice(Integer id) {
		Advice advice = (Advice) sessionFactory.getCurrentSession().load(
				Advice.class, id);
		if (null != advice) {
			logger.info("Delete advice: " + advice);
			long startTime = System.nanoTime();
			sessionFactory.getCurrentSession().delete(advice);
			long endTime = System.nanoTime();
			long duration = (endTime - startTime) / 1000000;
			logger.info("Delete advice: " + advice + " Time: " + duration
					+ "ms");
		}
	}

	@Override
	public void updateAdvice(Advice advice) {
		logger.info("Update adviceId: " + advice);
		long startTime = System.nanoTime();
		sessionFactory.getCurrentSession().update(advice);
		long endTime = System.nanoTime();
		long duration = (endTime - startTime) / 1000000;
		logger.info("Update an advice Time: " + duration + "ms");
	}

	@Override
	@SuppressWarnings("unchecked")
	public List<Advice> listAdvice() {
		logger.info("List all advices");
		long startTime = System.nanoTime();
		List<Advice> result = sessionFactory
				.getCurrentSession()
				.createQuery(
						"select ad from Advice ad order by ad.created desc")
				.setCacheable(true).list();
		long endTime = System.nanoTime();
		long duration = (endTime - startTime) / 1000000;
		logger.info("List all advices Time: " + duration + "ms");
		return result;
	}

	@Override
	public Advice getAdvice(Integer id) {
		logger.info("Get adviceId: " + id);
		long startTime = System.nanoTime();
		Advice advice = (Advice) sessionFactory.getCurrentSession().get(
				Advice.class, id);
		long endTime = System.nanoTime();
		long duration = (endTime - startTime) / 1000000;
		logger.info("Get adviceId: " + id + " Time: " + duration + "ms");
		return advice;
	}

	@Override
	@SuppressWarnings("unchecked")
	public List<Advice> listDailyAdviceForService(Integer serviceId, Date date) {
		logger.info("list daily advice for serviceId: " + serviceId
				+ " where date is after: " + date);
		String hql = "select ad from Advice ad join ad.service ser where ser.serviceId = :serviceId and ad.created >= :date order by ad.created desc";
		long startTime = System.nanoTime();
		List<Advice> result = sessionFactory.getCurrentSession()
				.createQuery(hql).setParameter("serviceId", serviceId)
				.setParameter("date", date).list();
		Collections.sort(result, new Comparator<Advice>() {
			@Override
			public int compare(Advice o1, Advice o2) {
				return o1.getTimeAsDate().compareTo(o2.getTimeAsDate());
			}
		});
		long endTime = System.nanoTime();
		long duration = (endTime - startTime) / 1000000;
		logger.info("list daily advice for serviceId: " + serviceId
				+ " where date is after: " + date + " Time: " + duration + "ms");
		return result;
	}

	@Override
	@SuppressWarnings("unchecked")
	public List<Advice> searchForAdvices(String search) {
		long startTime = System.nanoTime();

		List<Advice> advices = new ArrayList<Advice>();
		String[] fields = { "course", "time", "bet", "betType",
				"bestEpBookieExchange", "finishingPosition", "wonOrLost",
				"lsAtEp", "lspAtBsp", "racecardNumber", "market", "type" };
		FullTextSession fullTextSession = Search
				.getFullTextSession(sessionFactory.getCurrentSession());
		boolean wildcard = (search.contains("*") || search.contains("?") ? true
				: false);
		QueryBuilder queryBuilder = fullTextSession.getSearchFactory()
				.buildQueryBuilder().forEntity(Advice.class).get();
		for (String s : fields) {
			org.apache.lucene.search.Query query;
			if (wildcard) {
				query = queryBuilder.keyword().wildcard().onField(s)
						.matching(search).createQuery();
			} else {
				query = queryBuilder.keyword().onField(s).matching(search)
						.createQuery();
			}
			Query fullTextQuery = fullTextSession.createFullTextQuery(query,
					Advice.class);
			advices.addAll(fullTextQuery.list());
		}
		long endTime = System.nanoTime();
		long duration = (endTime - startTime) / 1000000;
		logger.info("Search For Advices with string: " + search + " Time: "
				+ duration + "ms");
		Collections.sort(advices, new AdviceComparator());
		return advices;
	}

	@Override
	@SuppressWarnings("unchecked")
	public List<Advice> listAdvicesForService(Service service, Date startDate,
			Date endDate) {
		logger.info("List advices for serviceId: " + service.getServiceId()
				+ " between the dates: " + startDate + " and " + endDate);
		String hql = "select ad from Advice ad join ad.service ser where ser.serviceId = :serviceId and :startDate <= ad.created  and ad.created <= :endDate order by ad.created desc";
		long startTime = System.nanoTime();
		List<Advice> result = sessionFactory.getCurrentSession()
				.createQuery(hql)
				.setParameter("serviceId", service.getServiceId())
				.setParameter("startDate", startDate)
				.setParameter("endDate", endDate).list();
		long endTime = System.nanoTime();
		long duration = (endTime - startTime) / 1000000;
		logger.info("List advices for serviceId: " + service.getServiceId()
				+ " between the dates: " + startDate + " and " + endDate
				+ " Time: " + duration + "ms");
		return result;
	}

	@Override
	@SuppressWarnings("unchecked")
	public List<Advice> listAllAdvicesForService(Service service) {
		logger.info("List all advices for serviceId: " + service.getServiceId());
		String hql = "select ad from Advice ad join ad.service ser where ser.serviceId = :serviceId order by ad.created desc";
		long startTime = System.nanoTime();
		List<Advice> result = sessionFactory.getCurrentSession()
				.createQuery(hql)
				.setParameter("serviceId", service.getServiceId())
				.setCacheable(true).list();
		long endTime = System.nanoTime();
		long duration = (endTime - startTime) / 1000000;
		logger.info("List all advices for serviceId: " + service.getServiceId()
				+ " Time: " + duration + "ms");
		return result;
	}
}
