package uk.co.trinitylogic.skyfall.dao;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.Criteria;
import org.hibernate.FetchMode;
import org.hibernate.Query;
import org.hibernate.SessionFactory;
import org.hibernate.criterion.Restrictions;
import org.hibernate.search.FullTextSession;
import org.hibernate.search.Search;
import org.hibernate.search.query.dsl.QueryBuilder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import uk.co.trinitylogic.skyfall.form.Member;
import uk.co.trinitylogic.skyfall.form.MemberComparator;
import uk.co.trinitylogic.skyfall.form.Service;

@Repository
@Transactional
public class MemberDAOImpl implements MemberDAO {

	protected final Log logger = LogFactory.getLog(getClass());

	@Autowired
	private SessionFactory sessionFactory;

	@Override
	public void addMember(Member member) {
		logger.info("Add member: " + member);
		long startTime = System.nanoTime();
		sessionFactory.getCurrentSession().save(member);
		long endTime = System.nanoTime();
		long duration = (endTime - startTime) / 1000000;
		logger.info("Added a new member: " + member + " Time: " + duration + "ms");
	}

	@Override
	public void removeMember(Integer id) {
		Member member = (Member) sessionFactory.getCurrentSession().load(Member.class, id);
		if (null != member) {
			logger.info("Delete member: " + member);
			long startTime = System.nanoTime();
			sessionFactory.getCurrentSession().delete(member);
			long endTime = System.nanoTime();
			long duration = (endTime - startTime) / 1000000;
			logger.info("Delete member: " + member + " Time: " + duration + "ms");
		}
	}

	@Override
	public void updateMember(Member member) {
		logger.info("Update member: " + member.getMemberId());
		long startTime = System.nanoTime();
		sessionFactory.getCurrentSession().update(member);
		long endTime = System.nanoTime();
		long duration = (endTime - startTime) / 1000000;
		logger.info("Update member: " + member.getMemberId() + " Time: " + duration + "ms");
	}

	@Override
	@SuppressWarnings("unchecked")
	public List<Member> listMember() {
		logger.info("List all members");
		String hql = "select mem from Member mem order by mem.surname, mem.forename";
		long startTime = System.nanoTime();
		List<Member> result = sessionFactory.getCurrentSession().createQuery(hql).setCacheable(true).list();
		long endTime = System.nanoTime();
		long duration = (endTime - startTime) / 1000000;
		logger.info("List all members Time: " + duration + "ms");
		return result;
	}

	@Override
	@SuppressWarnings("unchecked")
	public List<Member> listMembersForService(Service service) {
		logger.info("List all members for serviceId: " + service.getServiceId());
		String hql = "select mem from Member mem join mem.service ser where ser.serviceId = :serviceId order by mem.surname, mem.forename";
		long startTime = System.nanoTime();
		List<Member> result = sessionFactory.getCurrentSession().createQuery(hql).setParameter("serviceId", service.getServiceId()).setCacheable(true).list();
		long endTime = System.nanoTime();
		long duration = (endTime - startTime) / 1000000;
		logger.info("List all members Time: " + duration + "ms");
		return result;
	}

	@SuppressWarnings("deprecation")
	@Override
	public Member getMember(Integer id) {
		logger.info("Get member for memberId: " + id);
		long startTime = System.nanoTime();

		Criteria crit = sessionFactory.getCurrentSession().createCriteria(Member.class);
		crit.add(Restrictions.eq("memberId", id));
		crit.setFetchMode("service", FetchMode.EAGER);
		Member member = (Member) crit.uniqueResult();

		// Member member = (Member)
		// sessionFactory.getCurrentSession().get(Member.class, id);
		long endTime = System.nanoTime();
		long duration = (endTime - startTime) / 1000000;
		logger.info("Get member for memberId: " + id + " Time: " + duration + "ms");
		return member;
	}

	@Override
	@SuppressWarnings({ "unchecked", "deprecation" })
	public Member getMember(String email) {
		logger.info("Get member for email: " + email);
		long startTime = System.nanoTime();

		// String hql =
		// "select mem from Member mem join mem.service ser where mem.email1 =:email";

		Criteria crit = sessionFactory.getCurrentSession().createCriteria(Member.class);
		crit.add(Restrictions.eq("email1", email));
		crit.setFetchMode("service", FetchMode.EAGER);
		List<Member> member = crit.list();

		// List<Member> member = sessionFactory.getCurrentSession()
		// .createQuery(hql).setParameter("email", email)
		// .setCacheable(true).list();
		long endTime = System.nanoTime();
		long duration = (endTime - startTime) / 1000000;
		logger.info("Get member for email: " + email + " Time: " + duration + "ms");
		if (member.size() > 0) {
			return member.get(0);
		} else {
			return null;
		}
	}

	@Override
	@SuppressWarnings("unchecked")
	public List<Member> listMembersForMailServiceOptIn(Service service) {
		logger.info("list members for mail service opt-in");
		String hql = "select mem from Member mem join mem.service ser where mem.email1OptInBets=true and mem.email1 != '' and ser.serviceId = :serviceId order by mem.surname, mem.forename";
		long startTime = System.nanoTime();
		List<Member> result = sessionFactory.getCurrentSession().createQuery(hql).setParameter("serviceId", service.getServiceId()).setCacheable(true).list();
		long endTime = System.nanoTime();
		long duration = (endTime - startTime) / 1000000;
		logger.info("list members for mail service opt-in Time: " + duration + "ms");
		return result;
	}

	@Override
	@SuppressWarnings("unchecked")
	public List<Member> listMembersForTextServiceOptIn(Service service) {
		logger.info("list members for text service opt-in");
		String hql = "select mem from Member mem join mem.service ser where mem.mobileOptInForBets=true and mem.mobile != '' and ser.serviceId = :serviceId order by mem.surname, mem.forename";
		long startTime = System.nanoTime();
		List<Member> result = sessionFactory.getCurrentSession().createQuery(hql).setParameter("serviceId", service.getServiceId()).setCacheable(true).list();
		long endTime = System.nanoTime();
		long duration = (endTime - startTime) / 1000000;
		logger.info("list members for text service opt-in Time: " + duration + "ms");
		return result;
	}

	@Override
	@SuppressWarnings("unchecked")
	public List<Member> searchForMembers(String search) {
		List<Member> members = new ArrayList<Member>();
		String[] fields = { "title", "forename", "surname", "address1", "town", "county", "postcode", "country", "mobile", "telephoneHome", "email1" };
		FullTextSession fullTextSession = Search.getFullTextSession(sessionFactory.getCurrentSession());
		boolean wildcard = (search.contains("*") || search.contains("?") ? true : false);
		QueryBuilder queryBuilder = fullTextSession.getSearchFactory().buildQueryBuilder().forEntity(Member.class).get();
		for (String s : fields) {
			org.apache.lucene.search.Query query;
			if (wildcard) {
				query = queryBuilder.keyword().wildcard().onField(s).matching(search).createQuery();
			} else {
				query = queryBuilder.keyword().onField(s).matching(search).createQuery();
			}
			Query fullTextQuery = fullTextSession.createFullTextQuery(query, Member.class);
			members.addAll(fullTextQuery.list());
		}
		Collections.sort(members, new MemberComparator());
		return members;
	}

	@Override
	public Long memberCount() {
		logger.info("Count all members");
		String hql = "select count(*) from Member mem";
		long startTime = System.nanoTime();
		Long result = (Long) sessionFactory.getCurrentSession().createQuery(hql).setCacheable(true).uniqueResult();
		long endTime = System.nanoTime();
		long duration = (endTime - startTime) / 1000000;
		logger.info("Count all members Time: " + duration + "ms");
		return result;
	}

	@Override
	public Long memberCountForService(Service service) {
		logger.info("Count all members for serviceId: " + service.getServiceId());
		String hql = "select count(*) from Member mem join mem.service ser where ser.serviceId = :serviceId";
		long startTime = System.nanoTime();
		Long result = (Long) sessionFactory.getCurrentSession().createQuery(hql).setParameter("serviceId", service.getServiceId()).setCacheable(true).uniqueResult();
		long endTime = System.nanoTime();
		long duration = (endTime - startTime) / 1000000;
		logger.info("Count all members for serviceId: " + service.getServiceId() + " took: " + duration + "ms");
		return result;
	}

	@Override
	@SuppressWarnings("unchecked")
	public List<Member> listDistinctMembersForMailServiceOptIn(List<Service> services) {
		String insert = getServiceIdInsertHql(services);
		logger.info("list distinct members for mail service opt-in");
		String hql = "select mem from Member mem join mem.service ser where mem.email1OptInBets=true and mem.email1 != '' and " + insert + " group by mem.memberId order by mem.surname, mem.forename";
		long startTime = System.nanoTime();
		List<Member> result = sessionFactory.getCurrentSession().createQuery(hql).setCacheable(true).list();
		long endTime = System.nanoTime();
		long duration = (endTime - startTime) / 1000000;
		logger.info("list distinct members for mail service opt-in Time: " + duration + "ms");
		return result;
	}

	@Override
	@SuppressWarnings("unchecked")
	public List<Member> listDistinctMembersForTextServiceOptIn(List<Service> services) {
		String insert = getServiceIdInsertHql(services);
		logger.info("list distinct members for mail service opt-in");
		String hql = "select mem from Member mem join mem.service ser where mem.mobileOptInForBets=true and  mem.mobile != '' and " + insert
				+ " group by mem.memberId order by mem.surname, mem.forename";
		long startTime = System.nanoTime();
		List<Member> result = sessionFactory.getCurrentSession().createQuery(hql).setCacheable(true).list();
		long endTime = System.nanoTime();
		long duration = (endTime - startTime) / 1000000;
		logger.info("list distinct members for mail service opt-in Time: " + duration + "ms");
		return result;
	}

	@Override
	@SuppressWarnings("unchecked")
	public List<Member> listDistinctMembersForServiceList(List<Service> services) {
		String insert = getServiceIdInsertHql(services);
		logger.info("list distinct members for a service list");
		String hql = "select mem from Member mem join mem.service ser where " + insert + " group by mem.memberId order by mem.surname, mem.forename";
		long startTime = System.nanoTime();
		List<Member> result = sessionFactory.getCurrentSession().createQuery(hql).setCacheable(true).list();
		long endTime = System.nanoTime();
		long duration = (endTime - startTime) / 1000000;
		logger.info("list distinct members for a service list Time: " + duration + "ms");
		return result;
	}

	private String getServiceIdInsertHql(List<Service> services) {
		StringBuffer ids = new StringBuffer();
		for (Service s : services) {
			ids.append("ser.serviceId = ").append(s.getServiceId()).append(" or ");
		}
		String insert = ids.substring(0, ids.length() - 4);
		logger.info("HQL insert statment: " + insert);
		return insert;
	}

	@Override
	@SuppressWarnings("unchecked")
	public List<Member> listDistinctMembersForMailServiceDoubleOptIn(List<Service> services) {
		String insert = getServiceIdInsertHql(services);
		logger.info("list distinct members for mail service double opt-in");
		String hql = "select mem from Member mem join mem.service ser where mem.doubleOptIn=true and mem.email1 != '' and " + insert + " group by mem.memberId order by mem.surname, mem.forename";
		long startTime = System.nanoTime();
		List<Member> result = sessionFactory.getCurrentSession().createQuery(hql).setCacheable(true).list();
		long endTime = System.nanoTime();
		long duration = (endTime - startTime) / 1000000;
		logger.info("list distinct members for mail service double opt-in Time: " + duration + "ms");
		return result;
	}

}
