package uk.co.trinitylogic.skyfall.dao;

import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import uk.co.trinitylogic.skyfall.form.Advice;
import uk.co.trinitylogic.skyfall.form.Service;
import uk.co.trinitylogic.skyfall.service.AdviceService;
import uk.co.trinitylogic.skyfall.service.MemberService;

@Repository
@Transactional
public class ServiceDAOImpl implements ServiceDAO {

	protected final Log logger = LogFactory.getLog(getClass());

	@Autowired
	private SessionFactory sessionFactory;

	@Autowired
	private MemberService memberService;

	@Autowired
	private AdviceService adviceService;

	@Override
	public void addService(Service service) {
		logger.info("Add service: " + service);
		long startTime = System.nanoTime();
		sessionFactory.getCurrentSession().save(service);
		long endTime = System.nanoTime();
		long duration = (endTime - startTime) / 1000000;
		logger.info("Added a new service: " + service + " Time: " + duration
				+ "ms");
	}

	@Override
	public Service getService(Integer id) {
		logger.info("Get service for serviceId: " + id);
		long startTime = System.nanoTime();
		Service service = (Service) sessionFactory.getCurrentSession().get(
				Service.class, id);
		long endTime = System.nanoTime();
		long duration = (endTime - startTime) / 1000000;
		logger.info("Get service for serviceId: " + id + " Time: " + duration
				+ "ms");
		return service;
	}

	@Override
	public void removeService(Integer id) {
		Service service = (Service) sessionFactory.getCurrentSession().load(
				Service.class, id);
		if (null != service) {
			long startTime = System.nanoTime();
			List<Advice> advices = adviceService
					.listAllAdvicesForService(service);
			logger.info("First delete all [" + advices.size()
					+ "] advices for the service: " + service);
			for (Advice advice : advices) {
				sessionFactory.getCurrentSession().delete(advice);
			}
			long endTime = System.nanoTime();
			long duration = (endTime - startTime) / 1000000;
			logger.info("Delete all advices: " + service + " Time: " + duration
					+ "ms");
			logger.info("Now delete the service: " + service);
			startTime = System.nanoTime();
			sessionFactory.getCurrentSession().delete(service);
			endTime = System.nanoTime();
			duration = (endTime - startTime) / 1000000;
			logger.info("Delete the service: " + service + " Time: " + duration
					+ "ms");
		}
	}

	@Override
	public void updateService(Service service) {
		logger.info("Update service: " + service);
		long startTime = System.nanoTime();
		sessionFactory.getCurrentSession().update(service);
		long endTime = System.nanoTime();
		long duration = (endTime - startTime) / 1000000;
		logger.info("Update service: " + service + " Time: " + duration + "ms");
	}

	@Override
	@SuppressWarnings("unchecked")
	public List<Service> listService() {
		logger.info("List services");
		long startTime = System.nanoTime();
		String hql = "select ser from Service ser where ser.hidden = false";

		Session session = sessionFactory.openSession();
		session.beginTransaction();

		List<Service> result = session.createQuery(hql).setCacheable(true)
				.list();

		session.getTransaction().commit();
		session.close();

		long endTime = System.nanoTime();
		long duration = (endTime - startTime) / 1000000;
		logger.info("List services Time: " + duration + "ms");
		return result;
	}

	@Override
	@SuppressWarnings("unchecked")
	public List<Service> listServiceWithMemberCount() {
		logger.info("List services with member count");
		long startTime = System.nanoTime();
		String hql = "select ser from Service ser where ser.hidden = false";
		List<Service> result = sessionFactory.getCurrentSession()
				.createQuery(hql).setCacheable(true).list();
		for (Service service : result) {
			Long members = memberService.memberCountForService(service);
			service.setMembers(members);
		}
		long endTime = System.nanoTime();
		long duration = (endTime - startTime) / 1000000;
		logger.info("List services with member count Time: " + duration + "ms");
		return result;
	}

	@Override
	@SuppressWarnings("unchecked")
	public List<Service> listLiveService() {
		logger.info("List all live services");
		String hql = "select ser from Service ser where ser.live = 1";
		long startTime = System.nanoTime();
		List<Service> result = sessionFactory.getCurrentSession()
				.createQuery(hql).setCacheable(true).list();
		long endTime = System.nanoTime();
		long duration = (endTime - startTime) / 1000000;
		logger.info("List all live services Time: " + duration + "ms");
		return result;
	}

	@Override
	public void hideService(Integer id) {
		Service service = (Service) sessionFactory.getCurrentSession().load(
				Service.class, id);
		if (null != service) {
			logger.info("Hide service: " + service);
			service.setHidden(true);
			long startTime = System.nanoTime();
			sessionFactory.getCurrentSession().update(service);
			long endTime = System.nanoTime();
			long duration = (endTime - startTime) / 1000000;
			logger.info("Hide service: " + service + " Time: " + duration
					+ "ms");
		}
	}
}
